checkRequire("Lua_ArtifactLineupPanel");
checkRequire("Lua_HeroLineupCell");
Lua_BaseLineupWindow = class("Lua_BaseLineupWindow", Lua_Window);
function Lua_BaseLineupWindow:ctor()
self:superCall("ctor");
    
    self.layer = MainWinDefine.NORMAL_WINDOW_LAYER;
	self.isFullSizeWin = true;
	self.isSingleShowWin = true;
	self.showBackBtn = true;

    self.lineup_type = Lua_HeroUtil.LINEUP_MAIN_TYPE;
    self.default_camp = Lua_HeroUtil.All_TYPE;
    self.positionNum = 5; -- 站位数量
	self.teamIdx = 1; -- 分组索引
	self.stand_type = 1; -- 站位类型
end

function Lua_BaseLineupWindow:buildShowItem()
	self:superCall("buildShowItem");

    if DataCenter.funcOpenData:isOpened(10210) then
        self.artifactPanel = Lua_ArtifactLineupPanel.new(self.bottomObject);
        self.artifactPanel:setLocation(-282, 620.5, 0);
    end

    local siftAllInfo = DataCenter.heroUtil.siftAllInfo;
    self.campPosXArr = Array();
    self.campArr = Array();
    self.siftSpace = 82;
    local len = siftAllInfo:getSize();
    local item;
    local w = -(self.siftSpace * len) / 2 + self.siftSpace / 2;
    local posX;
    for i = 1, len do
        item = Lua_CampSiftBaseItem.new(self.bottomObject, self.siftSpace);
        
        posX = ((i - 1) % len) * self.siftSpace + w;
        item:setLocation(posX, 157, 0);

        item:setData(siftAllInfo[i]);
        item:setClickCallBack(self, self.onCampSelected);

        self.campPosXArr:add(posX);
        self.campArr:add(item);
    end

    self.heroList = Lua_DragList.new(self.bottomObject, 632, 290, Lua_AxisDefine.Vertical);
    self.heroList:setLocation(0, 375.5);
    self.heroList.canLongPress = true;
    self.heroList.longPressLimit = 0.4;
    self.heroList.tweenId = 2006;
    self.heroList:setListCell(Lua_HeroLineupCell, 112, 112, 5);
    self.heroList:setCellSpacing(18, 22);

    if self.restrain_btn ~= nil then
        self.restrain_btn:addClickFuntion(self, self.onRestrainBtnClick);
    end
    if self.stand_btn ~= nil then
        self.stand_btn:addClickFuntion(self, self.onBtnStandClick);
    end

    self:addLuaEvent("LINEUP_ARTIFACE_SELECT", self, self.onArtifactSelected);
    self:addLuaEvent("LINEUP_HERO_SELECT", self, self.onHeroSelected); -- 选择英雄
    self:addLuaEvent("LINEUP_STANDTYPE_SELECT", self, self.onStandTypeSelected); -- 选择站位更新布阵
    self:addCSharpEvent("CAMP_POSITION_CHANGE", self, self.onHeroCampPosChange); -- 场景移动英雄更新站位
end

function Lua_BaseLineupWindow:dispose()
    if self.sceneDic ~= nil then
        for k, v in pairs(self.sceneDic) do
            Scene3DManager.instance:destroyScene(k, 0);
        end
        self.sceneDic = nil;
        self.winSceneId = 0;
    end
    if self.artifactPanel ~= nil then
        self.artifactPanel:dispose();
        self.artifactPanel = nil;
    end
    if self.campArr ~= nil then
        self.campArr:dispose();
        self.campArr:clear();
        self.campArr = nil;
    end
    if self.campPosXArr ~= nil then
        self.campPosXArr:clear();
        self.campPosXArr = nil;
    end
    if self.heroList ~= nil then
        self.heroList:dispose();
        self.heroList = nil;
    end
    if self.posEffIdsArr ~= nil then
        self.posEffIdsArr:clear();
        self.posEffIdsArr = nil;
    end
    self.modelIdsDic = nil;
    self.serverLineupDic = nil;
    self.localLineupDic = nil;
    self.serverArtifactDic = nil;
    self.localArtifactDic = nil;
    DataCenter.heroLineupData:setLocalLineupInfo(nil, nil);
    self:superCall("dispose");
end

-- 布阵场景
function Lua_BaseLineupWindow:showSceneByLineupType(lineup_type)
    if lineup_type == nil then
        return;
    end

    local templateId = Lua_HeroLineupCofingData.getConfigData(lineup_type).templateId;
    self:showSceneByTemplateId(templateId);
end

-- 布阵场景
function Lua_BaseLineupWindow:showSceneByTemplateId(template_id)
    if template_id == nil then
        return;
    end

    if self.sceneDic == nil then
        self.sceneDic = {};
    end
    local sceneId = DataCenter.heroLineupData:getFightLineupSceneId(template_id);
    self.sceneDic[sceneId] = 1;
    self:setWinSceneId(sceneId);
end

-- 更新布阵信息
function Lua_BaseLineupWindow:updateLineupInfo()
    self.serverLineupDic = DataCenter.heroLineupData:getServerLineupDic(self.lineup_type);
    self.localLineupDic = DataCenter.heroLineupData:getLocalLineupDic(self.serverLineupDic);

    self.serverArtifactDic = DataCenter.heroLineupData:getServerArtifactDic(self.lineup_type);
    self.localArtifactDic = DataCenter.heroLineupData:getLocalArtifactDic(self.serverArtifactDic);

    DataCenter.heroLineupData:setLocalLineupInfo(self.localLineupDic, self.localArtifactDic);

    self.heroList.customParam = self.localLineupDic;

    -- 站位阵型
    self.stand_type = DataCenter.heroLineupData:getServerStandType(self.lineup_type, self.teamIdx);
    self.tag_stand = self.stand_type;--记录服务端阵型
    
    if self.stand_btn ~= nil then
        local data = DataCenter.heroLineupUtil:getStandInfoByType(self.stand_type);
        self.stand_btn:setAltas("hero_altas_1", data.stand_icon);
    end

    self.sceneStandType = nil;
    self:updateScenePosEff(self.stand_type);
    self:updateSceneModel();
    self:updateArtifact();
    self:updateLineupNum();
    self:updateAureoleInfo();

    -- 引导
    self:checkWinGuide();
    Lua_MsgUtil.callFunc("LineUp_LocationInfo_Update", self.lineup_type);
end

-- 更新站位特效
function Lua_BaseLineupWindow:updateScenePosEff(standType)
    if self.winSceneId == nil or self.winSceneId <= 0 then
        return;
    end

    if self.sceneStandType == standType then
        return;
    end
    self.sceneStandType = standType;

    if self.posEffIdsArr == nil then
        self.posEffIdsArr = Array();
    end
    
    local campposId = Lua_HeroLineupCofingData.getConfigData(self.lineup_type).campposId;
    local list = DataCenter.heroLineupUtil:getStandInfoByType(standType).position_list;
    self.positionNum = #list;
    local idx;
    local data;
    self.showIdxList = {};--展示的站位
    for i = 1, self.positionNum do
        idx = list[i];
        self.showIdxList[idx] = 1;
        if ThingUtil.instance:hasThingById(self.winSceneId, 0, idx) == true then
            ThingUtil.instance:showPlayer(self.winSceneId, 0, idx);
        else
            data = SceneEffectData();
            data.roleId = idx;
            data.sceneId = self.winSceneId;
            data.kind = Lua_ThingDefine.SCENE_EFFECT;
            data.px = Lua_FightCampPosConfig.getConfigData(campposId):getPosxByIdx(Lua_FightDefine.FIGHT_ATTACKER, idx) - 10;
            data.py = 0;
            data.pz = Lua_FightCampPosConfig.getConfigData(campposId):getPoszByIdx(Lua_FightDefine.FIGHT_ATTACKER, idx) + 40;
            data.canBeClick = false;
            data.effectFile = "arena_caidai";
            data.effectName = "arena_caidai";
            data.showShadow = false;
            data.isShowName = false;
            data.toLand = false;
            data.rotX = 21;
            data.rotY = 0;
            data.rotZ = 0;
            data.scaX = 0.005;
            data.scaY = 0.005;
            data.scaZ = 0.005;
            ThingUtil.instance:addSceneEffect(data, true);
            self.posEffIdsArr:add(idx);
        end
    end
    self:hidePosEff()
end

function Lua_BaseLineupWindow:hidePosEff()
    if self.posEffIdsArr == nil or self.showIdxList == nil then
        return;
    end

    local len = self.posEffIdsArr:getSize();
    local roleId;
    for i = 1, len do
        roleId = self.posEffIdsArr[i];
        if self.showIdxList[roleId] == nil then
            ThingUtil.instance:hidePlayer(self.winSceneId, 0, roleId);
        end
    end
end

function Lua_BaseLineupWindow:onWinSceneLoaded(sceneId)
    if sceneId == self.winSceneId then
        self:hidePosEff();
    end
end

function Lua_BaseLineupWindow:updateSceneModel()

end

function Lua_BaseLineupWindow:showSceneModel(tab)
    if self.winSceneId == nil or self.winSceneId <= 0 then
        return;
    end

    if tab == nil then
        if self.modelIdsDic ~= nil and self.modelIdsDic[self.winSceneId] ~= nil then
            self.modelIdsDic[self.winSceneId] = nil;
        end
        ThingUtil.instance:removeAll(self.winSceneId, 0, Lua_ThingDefine.HERO);
        return;
    end

    if self.modelIdsDic == nil then
        self.modelIdsDic = {};
    end
    if self.modelIdsDic[self.winSceneId] == nil then
        self.modelIdsDic[self.winSceneId] = {};
    end

    local campposId = Lua_HeroLineupCofingData.getConfigData(self.lineup_type).campposId;
    local list = {};
    local uid;
    local data;
    local modelPosX;
    local modelPosY;
    local modelPosZ;
    local heroModel;
    for stand, heroVo in pairs(tab) do
        uid = heroVo.unique_id;
        list[uid] = 1;
        self.modelIdsDic[self.winSceneId][uid] = 1;

        modelPosX = Lua_FightCampPosConfig.getConfigData(campposId):getPosxByIdx(Lua_FightDefine.FIGHT_ATTACKER, stand);
        modelPosY = 0;
        modelPosZ = Lua_FightCampPosConfig.getConfigData(campposId):getPoszByIdx(Lua_FightDefine.FIGHT_ATTACKER, stand);
        heroModel = DataCenter.thingData:getThingPlayer(self.winSceneId, 0, uid);

        if heroModel ~= nil then
            heroModel.player:setPosPoint(modelPosX, modelPosY, modelPosZ, true);
            data = heroModel.player.data;
            data.formationPos = stand;
            data.campPosId = 0;
            data.standType = self.sceneStandType;
            data:refreshShowPos();
        else
            data = HeroData();
            data.kind = Lua_ThingDefine.HERO;
            data.isShowName = true;
            data.isShowHpBar = false;
            data.isShowBuffIcon = false;
            data.canBeClick = true;
            data.roleId = uid;
            data.objId = heroVo.id;
            data.formationPos = stand;
            data.campPosId = 0;
            data.standType = self.sceneStandType;
            data.lvl = heroVo.level;
            data.hp = 1;
            data.maxHp = 1;
            data.sceneId = self.winSceneId;
            data.fashion_id = tonumber(tostring(heroVo.clothes_id));
            data.dir = 90;
            data.px = modelPosX;
            data.py = modelPosY;
            data.pz = modelPosZ;
            data.canDrag = true;
            data.showShadow = true;
            data.bPlanarShadow = true;
            data.toLand = false;
            data.isUIHeadInfo = true;
            data:refreshShowPos();
            ThingUtil.instance:addAnimal(data, true);
        end
    end

    local dic = self.modelIdsDic[self.winSceneId];
    for rid, val in pairs(dic) do
        if list[rid] == nil then--存在和当前阵容不一样的UID就移除掉
            dic[rid] = nil;
            ThingUtil.instance:removeThing(self.winSceneId, 0, rid);
        end
    end
end

-- 更新神器
function Lua_BaseLineupWindow:updateArtifact()
    if self.artifactPanel == nil then
        return;
    end
    self.artifactPanel:setData(self.localArtifactDic, self.teamIdx);
end

-- 更新上阵数量
function Lua_BaseLineupWindow:updateLineupNum()

end

-- 更新己方光环
function Lua_BaseLineupWindow:updateAureoleInfo()

end

function Lua_BaseLineupWindow:onArtifactSelected(artifactId)
    local id = self.localArtifactDic[self.teamIdx];
    if id == artifactId then
        self.localArtifactDic[self.teamIdx] = 0;
    else
        self.localArtifactDic[self.teamIdx] = artifactId;
        OperateAlert.showLangValue(126317);
    end
    self:updateArtifact();
    self:updateAureoleInfo();
end

-- 更新阵营选择
function Lua_BaseLineupWindow:onCampSelected(campType, isUpdate)
    if self.selectedCampType == campType and isUpdate ~= true then
        return;
    end
    self.selectedCampType = campType;

    local posX;
    local item;
    for i = 1, self.campArr:getSize() do
        posX = self.campPosXArr[i];
        item = self.campArr[i];
        if item.data.type > campType then -- 往右移
            item:setLocation(posX + self.siftSpace / 2, item.y, 0);
        else -- 往左移
            item:setLocation(posX - self.siftSpace / 2, item.y, 0);
        end

        if item.data.type == campType then
            item:setSelectEnable(true);
        else
            item:setSelectEnable(false);
        end
    end
    
    self:updateHeroList();
end

function Lua_BaseLineupWindow:updateHeroList()
    local arr = DataCenter.heroLineupData:getHeroList(self.selectedCampType, self.lineup_type, self.teamIdx);
    self.heroList:setArrayData(arr, false);
    self.heroList:moveToIndex(1);
    self:showNone(arr == nil or arr:getSize() <= 0);
end

function Lua_BaseLineupWindow:showNone(bShow)
    if bShow then
        if self.noneIcon == nil then
            self.noneIcon = UICreate.createUIAtlasImage(self.bottomObject, "common_altas_3", "ui_liaotian_widget_002", 0, 375, 648, 119);
        end
        self.noneIcon:setEnable(true);

        if self.noneText == nil then
            self.noneText = UICreate.createLabel(self.bottomObject, LanUtil.getValue(126318), 0, 375, 100, 40, 34);
            self.noneText.pivotValue = 4;
            self.noneText.fontName = Lua_MyUISettings.huakangFontName;
            self.noneText.alignmentValue = 4;
            self.noneText.horizontalOverflowValue = 1;
            self.noneText.colorStr = Lua_ColorDefine.transColorStr("9C8472");
        end
        self.noneText:setEnable(true);
    else
        if self.noneIcon ~= nil then
            self.noneIcon:setEnable(false);
        end
        if self.noneText ~= nil then
            self.noneText:setEnable(false);
        end
    end
end

function Lua_BaseLineupWindow:onHeroSelected(uniqueId)
    local teamIdx, standIdx = DataCenter.heroLineupData:getHeroTeamAndStand(self.localLineupDic, uniqueId);
    if standIdx == nil then -- 未上阵
        standIdx = DataCenter.heroLineupData:getNullStandIdx(self.stand_type, self.localLineupDic, self.teamIdx);
        if standIdx == nil then -- 上阵英雄已达上限
            OperateAlert.showLangValue(126300);
            return;
        end

        local isInLineup = DataCenter.heroLineupData:checkHeroInLineup(self.localLineupDic, uniqueId);
        if isInLineup == true then -- 不能同时上阵2个相同的英雄
            OperateAlert.showLangValue(126302);
            return;
        end

        self.localLineupDic[self.teamIdx][standIdx] = uniqueId;
    else -- 已上阵
        self.localLineupDic[self.teamIdx][standIdx] = nil;
    end

    self:updateHeroSelected();
    self:updateSceneModel();
    self:updateLineupNum();
    self:updateAureoleInfo();

    -- 引导
    self:checkWinGuide();
    Lua_MsgUtil.callFunc("LineUp_LocationInfo_Update", self.lineup_type);
end

function Lua_BaseLineupWindow:updateHeroSelected()
    local showList = self.heroList.showList;
    for i = 1, showList:getSize() do
        showList[i]:updateSelected();
    end
end

-- 选择站位
function Lua_BaseLineupWindow:onStandTypeSelected(stand_type)
    if self.stand_type == stand_type then
        return;
    end
    local prev_stand_type = self.stand_type;
    self.stand_type = stand_type;

    local data = DataCenter.heroLineupUtil:getStandInfoByType(stand_type);
    self.stand_btn:setAltas("hero_altas_1", data.stand_icon);

    DataCenter.heroLineupData:updateStandByType(self.localLineupDic, self.teamIdx, prev_stand_type, stand_type);
    self:updateScenePosEff(self.stand_type);
    self:updateSceneModel();

    -- 引导
    self:checkWinGuide();
    Lua_MsgUtil.callFunc("LineUp_LocationInfo_Update", self.lineup_type);
end

function Lua_BaseLineupWindow:onHeroCampPosChange(uid)
    uid = tonumber(tostring(uid));
    local heroModel = DataCenter.thingData:getThingPlayer(self.winSceneId, 0, uid);
    if heroModel == nil then
        return;
    end

    local new_stand = heroModel.data.formationPos;
    DataCenter.heroLineupData:updateStandChange(self.localLineupDic[self.teamIdx], uid, new_stand, self.lineup_type, self.winSceneId);
    self:updateHeroList();
    self:updateLineupNum();
    self:updateSceneModel();
    self:updateAureoleInfo();

    -- 引导
    self:checkWinGuide();
    Lua_MsgUtil.callFunc("LineUp_LocationInfo_Update", self.lineup_type);
end

-- 克制
function Lua_BaseLineupWindow:onRestrainBtnClick(go)
    Lua_WindowUtil.openWin("Lua_HeroRestrainWindow", Lua_WinParam.ACTION_OPEN);
end

-- 阵法
function Lua_BaseLineupWindow:onBtnStandClick(go)
    Lua_WindowUtil.openWin("Lua_HeroLineupStandWindow", Lua_WinParam.ACTION_OPEN, self.stand_type);
end



















-- 检测引导数据
function Lua_BaseLineupWindow:checkWinGuide()
    -- print("Lua_BaseLineupWindow:checkWinGuide()")
    self:superCall("checkWinGuide");
end

-- 引导同步
function Lua_BaseLineupWindow:updateGuide(guideId, info)
    -- print("Lua_BaseLineupWindow updateGuide", guideId, info == nil)
    if info == nil then
        return;
    end

    local operateVal = info.operateVal; -- 操作行为参数
    -- print("操作行为参数operateVal = ", operateVal);
    if operateVal == nil then
        return;
    end

    local heroId = tonumber(info.heroId);
    if operateVal == 1 then -- 上阵
        local lineIdx, standIdx = DataCenter.heroLineupData:getLineupHeroInfo(self.lineup_type, heroId);
        -- print("lineIdx", lineIdx, "standIdx", standIdx);
        if standIdx == nil then -- 找到背包列表的对应item
            
        else -- 已上阵完成
            DataCenter.guideCommonData.winGuideResult = 1;
            Lua_MsgUtil.callFunc("Guide_WinSync_Complete", guideId);
        end
    elseif operateVal == 2 then -- 交换位置
        local lineIdx, standIdx = DataCenter.heroLineupData:getLineupHeroInfo(self.lineup_type, heroId);
        if standIdx ~= nil then
            local stand = info.stand;
            if stand ~= nil then -- 目标位置
                if stand == standIdx then -- 完成
                    DataCenter.guideCommonData.winGuideResult = 1;
                    Lua_MsgUtil.callFunc("Guide_WinSync_Complete", guideId);
                else
                    -- 获取场景对应英雄
                    -- local uid = self.data:getHeroUidByStandIdx(lineupIdx, standIdx);
                    local uid = DataCenter.heroLineupData:getHeroUidByStandIdx(self.lineup_type, lineIdx, standIdx);
                    if uid > 0 then
                        DataCenter.guideCommonData.guideClickObjPath = DataCenter.guideUtil:getSceneObjPath(self.winSceneId, nil, uid);
                        DataCenter.guideCommonData.guideSceneId = self.winSceneId; -- 场景
                        -- print("获取场景对应英雄", self.winSceneId, DataCenter.guideCommonData.guideClickObjPath);
                        Lua_MsgUtil.callFunc("Guide_WinSync_Complete", guideId);
                    end
                end
            else -- 无条件,完成
                DataCenter.guideCommonData.winGuideResult = 1;
                Lua_MsgUtil.callFunc("Guide_WinSync_Complete", guideId);
            end
        end
    elseif operateVal == 3 then -- 英雄列表定位
        local delayCheckTime = DataCenter.guideUtil:getWinTweenTime(self.windowName) + 0.1;
        local param = {heroId = heroId, guideId = guideId};
        -- self:updateGuideDelayHeroLineUp(param);
        Lua_CommonUtil.delayCall(delayCheckTime, self, self.updateGuideDelayHeroLineUp, param);
    end
end

function Lua_BaseLineupWindow:updateGuideDelayHeroLineUp(param)
    local heroId, guideId = param.heroId, param.guideId;
    if heroId ~= nil then
        local lineIdx, standIdx = DataCenter.heroLineupData:getLineupHeroInfo(self.lineup_type, heroId);
        if standIdx ~= nil and standIdx > 0 then
            DataCenter.guideCommonData.winGuideResult = 1;
            Lua_MsgUtil.callFunc("Guide_WinSync_Complete", guideId);
        else
            local arr = self.heroList.showDragDataList;
            local heroCellId;
            local heroItem;
            local index;
            for i = 1, arr:getSize() do
                heroItem = arr:getItem(i).cell;
                heroCellId = heroItem.data and heroItem.data.id or 0;
                if heroCellId == heroId then
                    self.heroList.canDrag = false;
                    self.heroList.canLongPress = false;
                    DataCenter.guideCommonData.antiRepeatClick = false;
                    DataCenter.guideCommonData.guideClickObjTarget = heroItem.content;
                    Lua_MsgUtil.callFunc("Guide_WinSync_Complete", guideId);
                    break;
                end
            end
        end
    end
end