-- 边境远征 布阵
checkRequire("Lua_BaseLineupWindow");
Lua_ExpeditionLineupWindow = class("Lua_ExpeditionLineupWindow", Lua_BaseLineupWindow);
function Lua_ExpeditionLineupWindow:ctor()
	self:superCall("ctor");

	self.windowName = "Lua_ExpeditionLineupWindow";
	self.skinName = "FightLineupWindow_skin";
end

function Lua_ExpeditionLineupWindow:buildShowItem()
	self:superCall("buildShowItem");

    self.leftAureoleItem = Lua_HeroLineupAureoleItem.new(self.topObject, 1);
    self.leftAureoleItem:setLocation(-294, -44, 0);

    self.rightAureoleItem = Lua_HeroLineupAureoleItem.new(self.topObject, 2);
    self.rightAureoleItem:setLocation(294, -44, 0);

    self.heroList:setListCell(Lua_ExpeditionLineupHeroItem, 112, 126, 5);
    self.heroList:setCellSpacing(18, 13);

    self.btnFight:setLangValue(126305);
	self.btnFight:addClickFuntion(self, self.onBtnSaveClick);
	self.one_btn:setEnable(false);

	self:addLuaEvent("UPDATE_HERO_LINEUP_INFO", self, self.onLineupInfoUpdate); -- 阵容保存
end

function Lua_ExpeditionLineupWindow:dispose()
    if self.leftAureoleItem ~= nil then
        self.leftAureoleItem:dispose();
        self.leftAureoleItem = nil;
    end
    if self.rightAureoleItem ~= nil then
        self.rightAureoleItem:dispose();
        self.rightAureoleItem = nil;
    end
    if self.enemyLineupPanel ~= nil then
        self.enemyLineupPanel:dispose();
        self.enemyLineupPanel = nil;
    end
	self:superCall("dispose");
end

function Lua_ExpeditionLineupWindow:showWin(param)
	self.showData = param;
	
	local aureole_list = DataCenter.heroLineupData:getAureoleInfoByHeroList(param.msg.def_hero_list);
	local aureole_id = DataCenter.heroLineupUtil:getAureoleId(aureole_list);
	self:updateEnemyAureoleInfo(aureole_id, param.name, param.msg.def_player.force);
	self:updateEnemyLineup(2, param.msg.def_hero_list);

	self.lineup_type = Lua_HeroUtil.LINEUP_EXPEDITION;
    self:showSceneByLineupType(self.lineup_type);
    self:updateLineupInfo();
	self:updateHeroInfo()
end

function Lua_ExpeditionLineupWindow:onLineupInfoUpdate(lineup_type)
    self.serverLineupDic = DataCenter.heroLineupData:getServerLineupDic(lineup_type);
    self.serverArtifactDic = DataCenter.heroLineupData:getServerArtifactDic(lineup_type);
end

function Lua_ExpeditionLineupWindow:updateHeroInfo()
	self:onCampSelected(self.default_camp, true);
	self:updateSceneModel();
	self:updateAureoleInfo();
end

function Lua_ExpeditionLineupWindow:updateHeroList()
	local arr = self:getHeroList(self.selectedCampType, DataCenter.expeditionData.havaHeroArr);
	self.heroList:setArrayData(arr);
	self:showNone(arr == nil or arr:getSize() <= 0);
end

function Lua_ExpeditionLineupWindow:updateSceneModel()
	if self.localLineupDic == nil or self.teamIdx == nil then
		return;
	end

	local tab = {};
	local heroVo;
	for k, v in pairs(self.localLineupDic[self.teamIdx]) do
		heroVo = self:getLineupHero(v);
		if heroVo ~= nil then
			tab[k] = heroVo;
		end
	end

	self:showSceneModel(tab);
end

function Lua_ExpeditionLineupWindow:updateLineupNum()
    self.lineupNum = table.length(self.localLineupDic[self.teamIdx]);
    self.show_num.text = LanUtil.formateStr(126310, tostring(self.lineupNum), tostring(self.positionNum));
end

-- 己方光环更新
function Lua_ExpeditionLineupWindow:updateAureoleInfo()
    local standDic = self.localLineupDic[self.teamIdx];
    local force = self:calcLineupForce(standDic);
    local aureole_list = self:getAureoleInfo(standDic);
    local aureole_id = DataCenter.heroLineupUtil:getAureoleId(aureole_list);
    self.leftAureoleItem:setData(aureole_id, DataCenter.roleData.name, force);
end

-- 敌方光环更新
function Lua_ExpeditionLineupWindow:updateEnemyAureoleInfo(aureole_id, name, force)
    self.rightAureoleItem:setData(aureole_id, name, force);
end

-- 九宫格数据更新
function Lua_ExpeditionLineupWindow:updateEnemyLineup(dataType, list)
	if list == nil then
		return;
	end

	if self.enemyLineupPanel == nil then
        self.enemyLineupPanel = Lua_HeroGridViewPanel.new(self.content, 1, 2);
        self.enemyLineupPanel:setRectAnchor(1);
        self.enemyLineupPanel:setLocation(223, -200, 0);
        self.enemyLineupPanel:setCanClick(true);
	end
	if dataType == 1 then -- 配置表数据
		self.enemyLineupPanel:setDataByConfig(list[self.teamIdx]);
	elseif dataType == 2 then -- 服务端数据
		self.enemyLineupPanel:setDataByProto(list);
	end
end

function Lua_ExpeditionLineupWindow:onHeroSelected(uniqueId)
    local teamIdx, standIdx = DataCenter.heroLineupData:getHeroTeamAndStand(self.localLineupDic, uniqueId);
    if standIdx == nil then -- 未上阵
        standIdx = DataCenter.heroLineupData:getNullStandIdx(self.stand_type, self.localLineupDic, self.teamIdx);
        if standIdx == nil then -- 上阵英雄已达上限
            OperateAlert.showLangValue(126300);
            return;
        end

        local isInLineup = self:checkHeroInLineup(self.localLineupDic, uniqueId);
        if isInLineup == true then -- 不能同时上阵2个相同的英雄
            OperateAlert.showLangValue(126302);
            return;
        end

        self.localLineupDic[self.teamIdx][standIdx] = uniqueId;
    else -- 已上阵
        self.localLineupDic[self.teamIdx][standIdx] = nil;
    end

    self:updateHeroSelected();
    self:updateSceneModel();
    self:updateLineupNum();
    self:updateAureoleInfo();
end

function Lua_ExpeditionLineupWindow:onBackBtnClick(go)
	local isChange = DataCenter.heroLineupData:checkLineupIsChange(self.teamIdx, self.serverLineupDic, self.localLineupDic);
	if isChange == false then
		isChange = DataCenter.heroLineupData:checkArtifactIsChange(self.teamIdx, self.serverArtifactDic, self.localArtifactDic);
	end
	if isChange == true then
		self.needCloseWin = true;
		self:showTipsWindow();
	else
		self:hide();
	end
end

function Lua_ExpeditionLineupWindow:onBtnSaveClick(go)
	local isChange = DataCenter.heroLineupData:checkLineupIsChange(self.teamIdx, self.serverLineupDic, self.localLineupDic);
	if isChange == false then
		isChange = DataCenter.heroLineupData:checkArtifactIsChange(self.teamIdx, self.serverArtifactDic, self.localArtifactDic);
	end
	if isChange == true then
		self:onSureFun();
	else
		self:hide();
	end
end

-- 布阵改变提示
function Lua_ExpeditionLineupWindow:showTipsWindow()
    local tipData = { des = LanUtil.getValue(126306), posY = 30, btnType = 1, owner = self, cancelFun = self.onCancelFun, sureFun = self.onSureFun };
    Lua_WindowUtil.openWin("Lua_TipsWindow", Lua_WinParam.ACTION_OPEN, tipData);
end

function Lua_ExpeditionLineupWindow:onCancelFun()
	self:hide();
end

function Lua_ExpeditionLineupWindow:onSureFun()
    local configData = Lua_HeroLineupCofingData.getConfigData(self.lineup_type);
    if self.lineupNum == 0 and configData.can_nil == 0 then
        OperateAlert.showLangValue(126307);
        return;
    end

	if self.lineupNum < self.positionNum and DataCenter.expeditionData:checkPackHaveHero(self.localLineupDic, self.teamIdx) then
		local tipsData = {
			font_size = 28,
			des = LanUtil.getValue(126312), 
			btnType = 1,
			owner = self,
			sureFun = self.onSaveLineup
		};
		Lua_WindowUtil.openWin("Lua_TipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
		return;
	end

	self:onSaveLineup();
end

function Lua_ExpeditionLineupWindow:onSaveLineup()
    Lua_ProxyManager.getProxy("Lua_HeroProxy"):sHeroSaveLineup(self.lineup_type, self.stand_type, self.localLineupDic, self.localArtifactDic);
	-- if self.needCloseWin == true then
	-- 	self:hide();
	-- end
	self:hide();
end





function Lua_ExpeditionLineupWindow:getHeroList(campType, list)
	self.lineupHeroDic = {};
	local arr = Array();
	if list ~= nil then
		local heroVo;
		for i = 1, list:getSize() do
			heroVo = list[i];
			if campType == Lua_HeroUtil.All_TYPE or heroVo.campType == campType then
				arr:add(heroVo);
			end
			self.lineupHeroDic[heroVo.unique_id] = heroVo;
		end
	end
	arr:sortf(self, self.heroSortFun);
	return arr;
end

function Lua_ExpeditionLineupWindow:heroSortFun(a, b)
	local _a = a.blood / a.max_blood
	local _b = b.blood / b.max_blood
	if _a > 0 and _b > 0 then
		if a.lineup_id < b.lineup_id then
			return false
		elseif a.lineup_id == b.lineup_id then
			if a.heroForce < b.heroForce then
				return false
			end
		end
	elseif _a <= 0 and _b > 0 then
		return false
	elseif _a <= 0 and _b <= 0 then
		if a.heroForce < b.heroForce then
			return false
		end
	end
	return true
end

function Lua_ExpeditionLineupWindow:getLineupHero(uid)
	if self.lineupHeroDic == nil then
		return nil;
	end
	return self.lineupHeroDic[uid];
end

function Lua_ExpeditionLineupWindow:calcLineupForce(standDic)
	local force = 0;
	local heroVo;
	for k, v in pairs(standDic) do
		heroVo = self:getLineupHero(tonumber(v));
		if heroVo ~= nil then
			force = force + heroVo.heroForce;
		end
	end
    if self.localArtifactDic[self.teamIdx] > 0 then
        force = force + DataCenter.roleData.leaderForce;
    end
	return force;
end

function Lua_ExpeditionLineupWindow:getAureoleInfo(standDic)
	local tab = {};
	local heroVo;
	for k, v in pairs(standDic) do
		heroVo = self:getLineupHero(v);
		if heroVo ~= nil then
			if tab[heroVo.campType] == nil then
				tab[heroVo.campType] = 1;
			else
				tab[heroVo.campType] = tab[heroVo.campType] + 1;
			end
		end
	end
	return tab;
end

function Lua_ExpeditionLineupWindow:checkHeroInLineup(teamIdxDic, hero_uid)
	local newId = self:getLineupHero(hero_uid).id;
	local heroVo;
	for k, standDic in pairs(teamIdxDic) do
		for x, y in pairs(standDic) do
			heroVo = self:getLineupHero(y);
			if heroVo ~= nil then
				if heroVo.id == newId then
					return true;
				end
			end
		end
	end
	return false;
end




















Lua_ExpeditionLineupHeroItem = class("Lua_ExpeditionLineupHeroItem", Lua_HeroLineupCell);
function Lua_ExpeditionLineupHeroItem:buildShowItem()
	self:superCall("buildShowItem");

	self.itemIcon:setLocation(0, 7, 0);

	self.hpBar = UICreate.createProgressBar(self.content, "expedition_atlas_1", "ui_bjyz_progress_002_f", "expedition_atlas_1", "ui_bjyz_progress_002_b", 0, -63, 80, 8);
	self.hpBar:setBarOffset(2, 2);
	self.hpBar:setMaxValue(1);
	self.hpBar:setValue(0);
end

function Lua_ExpeditionLineupHeroItem:dispose()
	self.hpBar = nil;
	self:superCall("dispose");
end

function Lua_ExpeditionLineupHeroItem:setData(data)
	self.data = data;
	
	self.itemIcon:setData(data);
	if data.assist == 0 then
		self:showAssistIcon(false);
	else
		self:showAssistIcon(true);
	end
	if data.blood > 0 then
		self:showDeadIcon(false);
	else
		self:showDeadIcon(true);
	end

	self.hpBar:setMaxValue(data.max_blood);
	self.hpBar:setValue(data.blood);

	self:updateSelected();
end

function Lua_ExpeditionLineupHeroItem:showAssistIcon(bShow)
	if bShow then
		if self.assistIcon == nil then
			self.assistIcon = UICreate.createUIAtlasImage(self.content, "common_altas_3", "ui_bjyz_label_04", 39, -20.5, 0, 0);
		end
		self.assistIcon:setEnable(true);
	else
		if self.assistIcon ~= nil then
			self.assistIcon:setEnable(false);
		end
	end
end

function Lua_ExpeditionLineupHeroItem:showDeadIcon(bShow)
	if bShow then
		if self.deadIcon == nil then
			self.deadMask = UICreate.createUIAtlasImage(self.content, "common_altas_1", "ui_common_window_015", 0, 0, 0, 0);
			self.deadIcon = UICreate.createUIAtlasImage(self.content, "expedition_atlas_1", "ui_bjyz_label_05", 0, 6, 0, 0);
		end
		self.deadIcon:setEnable(true);
		self.deadMask:setEnable(true);
	else
		if self.deadIcon ~= nil then
			self.deadIcon:setEnable(false);
			self.deadMask:setEnable(false);
		end
	end
end

function Lua_ExpeditionLineupHeroItem:onPressClick()
	if self.data.blood <= 0 then
		OperateAlert.showLangValue(330037);
		return;
	end
    Lua_MsgUtil.callFunc("LINEUP_HERO_SELECT", self.data.unique_id);
end

function Lua_ExpeditionLineupHeroItem:onLongPressClick()
    if Lua_WindowUtil.isWindowShowing("Lua_HeroTipsWindow") == false then
		local heroVo = DataCenter.heroData:getItemByUniqueId(self.data.unique_id);
		if heroVo == nil then
			Lua_ProxyManager.getProxy("Lua_HeroProxy"):sLookRoleHeroInfo(self.data.unique_id);
		else
			Lua_WindowUtil.openWin("Lua_HeroTipsWindow", Lua_WinParam.ACTION_OPEN, heroVo);
		end
    end
end