-- 无穷战殿 团队布阵
checkRequire("Lua_FightLineupWindow");
Lua_InfinityHallTeamWindow = class("Lua_InfinityHallTeamWindow", Lua_FightLineupWindow);
function Lua_InfinityHallTeamWindow:ctor()
	self:superCall("ctor");

	self.windowName = "Lua_InfinityHallTeamWindow";
end

function Lua_InfinityHallTeamWindow:buildShowItem()
	self:superCall("buildShowItem");

    self.heroList:setListCell(Lua_InfinityHallTeamHeroItem, 112, 112, 5);
    self.one_btn:setEnable(false);
end

function Lua_InfinityHallTeamWindow:dispose()
	self:superCall("dispose");
end

function Lua_InfinityHallTeamWindow:showWin(param)
	self.showData = param;
	self.template_id = param.template_id;
	self.lineup_type = Lua_FightTemplateConfigData.getConfigData(self.template_id).lineup_type;
	
	self:showSceneByTemplateId(self.template_id);
	self:updateLineupInfo();
	self:showEnemyInfo(param.monster_player_id);
	self:onCampSelected(self.default_camp, true);
end

function Lua_InfinityHallTeamWindow:updateSceneModel()
	if self.localLineupDic == nil or self.teamIdx == nil then
		return;
	end

	local tab = {};
	local choose_hero;
	local heroVo;
	for k, v in pairs(self.localLineupDic[self.teamIdx]) do
		if DataCenter.infinityHallData:checkIsHelpOut(v) then
			choose_hero = DataCenter.infinityHallData.choose_hero;
			heroVo = Lua_HeroVo.new(choose_hero.hero_id);
			heroVo:fromServer(choose_hero);
		else
			heroVo = DataCenter.heroData:getItemByUniqueId(v);
		end
		tab[k] = heroVo;
	end

	self:showSceneModel(tab);
end

function Lua_InfinityHallTeamWindow:updateHeroList()
	local arr = self:getHeroList(self.selectedCampType);
	self.heroList:setArrayData(arr);
	self:showNone(arr == nil or arr:getSize() <= 0);
end

function Lua_InfinityHallTeamWindow:onHeroSelected(uniqueId)
    local teamIdx, standIdx = DataCenter.heroLineupData:getHeroTeamAndStand(self.localLineupDic, uniqueId);
    if standIdx == nil then -- 未上阵
        standIdx = DataCenter.heroLineupData:getNullStandIdx(self.stand_type, self.localLineupDic, self.teamIdx);
        if standIdx == nil then -- 上阵英雄已达上限
            OperateAlert.showLangValue(126300);
            return;
        end

        local isInLineup = DataCenter.infinityHallData:checkHeroInLineup(self.localLineupDic, uniqueId);
        if isInLineup == true then -- 不能同时上阵2个相同的英雄
            OperateAlert.showLangValue(126302);
            return;
        end

        self.localLineupDic[self.teamIdx][standIdx] = uniqueId;
    else -- 已上阵
        self.localLineupDic[self.teamIdx][standIdx] = nil;
    end

    self:updateHeroSelected();
    self:updateSceneModel();
    self:updateLineupNum();
    self:updateAureoleInfo();
end

-- 己方光环更新
function Lua_InfinityHallTeamWindow:updateAureoleInfo()
    local standDic = self.localLineupDic[self.teamIdx];
    local force = DataCenter.infinityHallData:calcLineupForce(standDic);
    if self.localArtifactDic[self.teamIdx] > 0 then
        force = force + DataCenter.roleData.leaderForce;
    end
    local aureole_list = DataCenter.infinityHallData:getAureoleInfo(standDic);
    local aureole_id = DataCenter.heroLineupUtil:getAureoleId(aureole_list);
    self.leftAureoleItem:setData(aureole_id, DataCenter.roleData.name, force);
end

function Lua_InfinityHallTeamWindow:onCheckLineup()
    if self.lineupNum == 0 then
        OperateAlert.showLangValue(126307);
        return;
    end

    if self.lineupNum < self.positionNum and DataCenter.infinityHallData:checkPackHaveHero(self.localLineupDic, self.teamIdx) then
        local tipsData = {
            des = LanUtil.getValue(126312),
            font_size = 28,
            posY = 30,
            btnType = 1,
            owner = self,
            sureFun = self.onStartFight
        };
        Lua_WindowUtil.openWin("Lua_TipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
        return;
    end

    self:onStartFight();
end

function Lua_InfinityHallTeamWindow:onStartFight()
	self:superCall("onStartFight");
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.InfinityHall_Fight, false, true)
end

function Lua_InfinityHallTeamWindow:getHeroList(campType)
	local arr = Array();
	local heroVo;

	-- 助阵英雄
	local choose_hero = DataCenter.infinityHallData.choose_hero;
	if choose_hero and tonumber(choose_hero.unique_id) > 0 then
		heroVo = Lua_HeroVo.new(choose_hero.hero_id);
		if campType == Lua_HeroUtil.All_TYPE or heroVo.campType == campType then
			heroVo:fromServer(choose_hero);
			heroVo.assist = 1;-- 0 不是援助，1 邀请援助
			arr:add(heroVo);
		end
	end

	-- 自己英雄
	local uidArr = DataCenter.heroData.allHeroUidArr;
	for i = 1, uidArr:getSize() do
		heroVo = DataCenter.heroData:getItemByUniqueId(uidArr[i]);
		if campType == Lua_HeroUtil.All_TYPE or heroVo.campType == campType then
			arr:add(heroVo);
		end
	end
	arr:sortf(self, self.heroListSortFun)
	return arr;
end

function Lua_InfinityHallTeamWindow:heroListSortFun(a, b)
	local _a
	local _b
	if a.assist == 1 then
		if self:checkHeroInLineup(a.unique_id) then
			_a = 10000000000 + a.heroForce
		else
			_a = a.heroForce
		end
	else
		_a = a:lineupSort(self.lineup_type, self.teamIdx)
	end
	if b.assist == 1 then
		if self:checkHeroInLineup(b.unique_id) then
			_b = 10000000000 + b.heroForce
		else
			_b = b.heroForce
		end
	else
		_b = b:lineupSort(self.lineup_type, self.teamIdx)
	end
	
	if _a < _b then
		return false
	elseif _a == _b then
		if a.star < b.star then
			return false
		elseif a.star == b.star then
			if a.level < b.level then
				return false
			elseif a.level == b.level then
				if a.color < b.color then
					return false
				elseif a.color == b.color then
					if a.sort > b.sort then
						return false
					end
				end
			end
		end
	end
	return true
end

function Lua_InfinityHallTeamWindow:checkHeroInLineup(uniqueId)
	for teamIdx, standDic in pairs(self.localLineupDic) do
		for standIdx, hero_uid in pairs(standDic) do
			if hero_uid == uniqueId then
				return true
			end
		end
	end
	return false
end




















Lua_InfinityHallTeamHeroItem = class("Lua_InfinityHallTeamHeroItem", Lua_HeroLineupCell);
function Lua_InfinityHallTeamHeroItem:buildShowItem()
	self:superCall("buildShowItem");
end

function Lua_InfinityHallTeamHeroItem:dispose()
	self:superCall("dispose");
end

function Lua_InfinityHallTeamHeroItem:setData(data)
	self.data = data;
	
	self.itemIcon:setData(data);
	self:showAssistHeroIcon(data.assist == 1);
	self:updateSelected();
end

function Lua_InfinityHallTeamHeroItem:showAssistHeroIcon(isShow)
	if isShow then
		if self.assistHeroIcon == nil then
			self.assistHeroIcon = UICreate.createUIAtlasImage(self.content, "common_altas_3", "ui_bjyz_label_04", 36, -33, 0, 0);
		end
		self.assistHeroIcon:setEnable(true);
	else
		if self.assistHeroIcon ~= nil then
			self.assistHeroIcon:setEnable(false);
		end
	end
end