--爬塔布阵
checkRequire("Lua_FightLineupWindow")
Lua_TowerLineupWindow = class("Lua_TowerLineupWindow", Lua_FightLineupWindow);
function Lua_TowerLineupWindow:ctor()
    self:superCall("ctor");
    self.windowName = "Lua_TowerLineupWindow";
end

function Lua_TowerLineupWindow:buildShowItem()
    self:superCall("buildShowItem");
    self.up_bg:setAltas("climb_tower_atlas_1", "ui_ttt_window_010");
    self.up_bg:SetDimensions(720, 57);
    self.up_bg:setLocation(0, -28.5, 0);

    self.leftAureoleItem:setLocation(-300, -44, 0);
    self.rightAureoleItem:setLocation(300, -44, 0);
    self.heroList:setListCell(Lua_TowerLineupCell, 112, 112, 5);

    self.vsIcon = UICreate.createUIAtlasImage(self.content, "common_altas_3", "ui_zd_icon_013", 0, -55, 0, 0);
    self.vsIcon.anchorValue = 1;

    self:addLuaEvent("TOWER_LINEUP_HERO_SELECT", self, self.onHeroSelected); -- 选择英雄
end

function Lua_TowerLineupWindow:showWin(param)
    self.showData = param;
    self.stage_id = param.cur_stage_id; -- 关卡id
    self.template_id = param.fight_id; -- 战斗模板id
    -- param.limit_camp; --上阵类型限制
    -- param.id; -- 爬塔类型

    local config = DataCenter.climbTowerUtil:getTowerConfig(self.stage_id);--关卡数据
    self.limit_num = config.limit_num; --最大上阵数量
    self.lineup_type = Lua_FightTemplateConfigData.getConfigData(self.template_id).lineup_type;

    self.selectCampDic = {}
    local obj;
    for i = 1, #param.limit_camp do
        obj = param.limit_camp[i];
        self.selectCampDic[obj] = i;
    end
    self.selectCampLength = table.length(self.selectCampDic)

    local item;
    local camp;
    for i = 1, self.campArr:getSize() do
        item = self.campArr[i];
        camp = item.data.type;
        item:setIconGray(self.selectCampLength > 0 and camp ~= 0 and self.selectCampDic[camp] == nil);
    end

    self:showSceneByTemplateId(self.template_id);
    self:updateLineupInfo();
    self:showEnemyInfo(config.robot);
    self:onCampSelected(Lua_HeroUtil.All_TYPE);
    self:updatePrompt();
end

function Lua_TowerLineupWindow:dispose()
    self.selectCampDic = nil;
    self.des = nil;
    self:superCall("dispose");
end

-- 更新布阵信息
function Lua_TowerLineupWindow:updateLineupInfo()
    self.serverLineupDic = DataCenter.heroLineupData:getServerLineupDic(self.lineup_type);
    self.localLineupDic = DataCenter.heroLineupData:getLocalLineupDic(self.serverLineupDic);

    self.serverArtifactDic = DataCenter.heroLineupData:getServerArtifactDic(self.lineup_type);
    self.localArtifactDic = DataCenter.heroLineupData:getLocalArtifactDic(self.serverArtifactDic);

    self.heroList.customParam = { lineupDic = self.localLineupDic, lineupType = self.lineup_type, towerId = self.showData.id };

    -- 站位类型
    self.stand_type = DataCenter.heroLineupData:getServerStandType(self.lineup_type, self.teamIdx);
    if self.stand_btn ~= nil then
        local data = DataCenter.heroLineupUtil:getStandInfoByType(self.stand_type);
        self.stand_btn:setAltas("hero_altas_1", data.stand_icon);
    end

    self.sceneStandType = nil;
    self:updateScenePosEff(self.stand_type);
    self:updateSceneModel();
    self:updateArtifact();
    self:updateLineupNum();
    self:updateAureoleInfo();
end

function Lua_TowerLineupWindow:updateLineupNum()
    self.lineupNum = table.length(self.localLineupDic[self.teamIdx]);
    self.show_num.text = LanUtil.formateStr(126310, tostring(self.lineupNum), tostring(self.limit_num));
    self:updatePrompt();
end

-- 更新条件描述
function Lua_TowerLineupWindow:updatePrompt()
    if self.selectCampLength == 0 then
        return
    end

    if self.checkHelp == nil then
        self.checkHelp = UICreate.createUIAtlasImage(self.topObject, "common_altas_1", "ui_common_button_016_n", -320, -215, 0, 0);
    end

    if self.promptText == nil then
        self.promptText = UICreate.createLabel(self.topObject, LanUtil.getValue(126322), -235, -215, 100, 31, 22);
        self.promptText.alignmentValue = 3;
        self.promptText.horizontalOverflowValue = 1;
    end

    self.promptText.text = LanUtil.formateStr(201024, tostring(self.limit_num), DataCenter.climbTowerUtil:limitCampName(self.showData.limit_camp));
    if self.lineupNum < self.limit_num then
        self.promptText.colorStr = Lua_ColorDefine.transColorStr("F63C3C")
    else
        self.promptText.colorStr = Lua_ColorDefine.transColorStr("59CD20")
    end
end

-- 更新筛选状态
function Lua_TowerLineupWindow:onCampSelected(campType)
    if self.selectCampLength > 0 and self.selectCampDic[campType] == nil and campType ~= 0 then
        OperateAlert.show(LanUtil.formateStr(201025, DataCenter.climbTowerUtil:limitCampName(self.showData.limit_camp)));
        return;
    end
    
    self:superCall("onCampSelected", campType, nil);
end

function Lua_TowerLineupWindow:updateHeroList()
    local arr;
    if self.selectCampLength > 0 and self.selectedCampType == 0 then
        arr = DataCenter.heroLineupData:getHeroListByCampDic(self.selectCampDic, self.lineup_type, self.teamIdx);
    else
        arr = DataCenter.heroLineupData:getHeroList(self.selectedCampType, self.lineup_type, self.teamIdx);
    end
    self.heroList:setArrayData(arr, false);
    self:showNone(arr == nil or arr:getSize() <= 0);
end

--选择英雄
function Lua_TowerLineupWindow:onHeroSelected(heroVo)
    local teamIdx, standIdx = DataCenter.heroLineupData:getHeroTeamAndStand(self.localLineupDic, heroVo.unique_id);
    if standIdx == nil then -- 未上阵
        standIdx = DataCenter.heroLineupData:getNullStandIdx(self.stand_type, self.localLineupDic, self.teamIdx);
        if standIdx == nil then -- 上阵英雄已达上限
            OperateAlert.showLangValue(126300);
            return;
        end

        local num = table.length(self.localLineupDic[self.teamIdx]);
        if num == self.limit_num then --上阵英雄已达上限
            OperateAlert.showLangValue(126300);
            return
        end

        local isInLineup = DataCenter.heroLineupData:checkHeroInLineup(self.localLineupDic, heroVo.unique_id);
        if isInLineup == true then -- 不能同时上阵2个相同的英雄
            OperateAlert.showLangValue(126302);
            return;
        end

        local trialLineupLimit = nil; -- 试用英雄上阵限制
        local trialTowerLimit = nil; -- 试用英雄爬塔次数限制
        if heroVo.trial_state then
            trialLineupLimit = DataCenter.heroUtil:trialHeroLineup(self.lineup_type);
            trialTowerLimit = DataCenter.heroData:isClimbLimit(heroVo.unique_id, self.showData.id);
        end
        
        if trialLineupLimit == true then
            if trialTowerLimit == false then
                OperateAlert.show(LanUtil.formateStr(201017, heroVo.name));
                return;
            elseif trialTowerLimit == true then
                local towerType = self.showData.id;
                local towerVo = DataCenter.climbTowerData:getTowerVoById(towerType);
                local count = DataCenter.heroData:getTrialTowerCount(heroVo.unique_id, towerType);
                local num = DataCenter.heroUtil:trialTowerlimit(towerType);
                OperateAlert.show(LanUtil.formateStr(201018, heroVo.name, towerVo.name, count, num));
            end
        end

        self.localLineupDic[self.teamIdx][standIdx] = heroVo.unique_id;
    else -- 已上阵
        self.localLineupDic[self.teamIdx][standIdx] = nil;
    end

    self:updateHeroSelected();
    self:updateSceneModel();
    self:updateLineupNum();
    self:updateAureoleInfo();
end

--一键上阵
function Lua_TowerLineupWindow:onOneBtnClick(go)
    local arr
    if self.selectCampLength == 0 then
        arr = DataCenter.heroLineupData:getHeroList(Lua_HeroUtil.All_TYPE, self.lineup_type, self.teamIdx, 1);
    else
        arr = DataCenter.heroLineupData:getHeroListByCampDic(self.selectCampDic, self.lineup_type, self.teamIdx, 1);
    end
    DataCenter.heroLineupData:setHeroOneKey(self.stand_type, self.localLineupDic, self.teamIdx, self.lineup_type, arr, self.limit_num);

    self:updateHeroSelected();
    self:updateSceneModel();
    self:updateLineupNum();
    self:updateAureoleInfo();
end

function Lua_TowerLineupWindow:onCheckLineup()
    if self.lineupNum == 0 then
        OperateAlert.showLangValue(126307);
        return
    end

    if self.lineupNum < self.limit_num then
        local arr
        if self.selectCampLength == 0 then
            arr = DataCenter.heroLineupData:getHeroList(Lua_HeroUtil.All_TYPE, self.lineup_type, self.teamIdx, 1);
        else
            arr = DataCenter.heroLineupData:getHeroListByCampDic(self.selectCampDic, self.lineup_type, self.teamIdx, 1);
        end
        if DataCenter.heroLineupData:checkPackHaveHero(self.localLineupDic, self.teamIdx, arr) then
            local tipsData = { 
                font_size = 28,
                des = LanUtil.getValue(126312),
                btnType = 1,
                owner = self,
                sureFun = self.onStartFight
             }
            Lua_WindowUtil.openWin("Lua_TipsWindow", Lua_WinParam.ACTION_OPEN, tipsData)
            return
        end
    end


    if self.selectCampLength > 0 then
        local standDic = self.localLineupDic[self.teamIdx];
        local campType;
        for standIdx, uid in pairs(standDic) do
            campType = DataCenter.heroData:getItemByUniqueId(uid).campType;
            if self.selectCampDic[campType] == nil then
                OperateAlert.showLangValue(201019);
                return;
            end
        end
    end

    self:onStartFight();
end

























Lua_TowerLineupCell = class("Lua_TowerLineupCell", Lua_HeroLineupCell);

function Lua_TowerLineupCell:setData(data)
    self.data = DataCenter.heroData:getItemByUniqueId(data);
    self.itemIcon:setData(self.data);

    self.trialLineupLimit = nil; -- 试用英雄上阵限制
    self.trialTowerLimit = nil; -- 试用英雄爬塔次数限制
    if self.data.trial_state then
        self.trialLineupLimit = DataCenter.heroUtil:trialHeroLineup(self.customParam.lineupType);
        self.trialTowerLimit = DataCenter.heroData:isClimbLimit(self.data.unique_id, self.customParam.towerId);
    end

    self:setLock(self.trialLineupLimit == false or (self.trialLineupLimit == true and self.trialTowerLimit == false));
    self:updateSelected();
end

function Lua_TowerLineupCell:updateSelected()
    local teamIdx, standIdx = DataCenter.heroLineupData:getHeroTeamAndStand(self.customParam.lineupDic, self.data.unique_id);
    self.itemIcon:setDrawMask(standIdx ~= nil);
end

function Lua_TowerLineupCell:onPressClick()
    if self.trialLineupLimit == false then
        OperateAlert.show(LanUtil.formateStr(201020, self.data.name));
        return false;
    end

    Lua_MsgUtil.callFunc("TOWER_LINEUP_HERO_SELECT", self.data);
    return true;
end
